#include <nagios/nagios.h>
#include <nagios/lib/kvvec.h>
#include "kvvec_json.h"

static char *expect_string( char **inptr, int *length );

/* Append to local temporary buffer */
#define APPEND(_CHR) do { \
	if (bufsize<=ptr+1) { \
		bufsize*=2; \
		if ((buf = (char *)realloc(buf, (size_t)bufsize)) == NULL) { \
			perror("realloc"); \
			my_free(buf); \
			return NULL; \
		} \
	} \
	buf[ptr++] = (_CHR); \
} while(0)

/* Rewind buffer */
#define REWIND ptr=0

char *kvvec_to_json( struct kvvec *kvv ) {
	char *buf = NULL;
	unsigned int ptr = (unsigned int)0;
	unsigned int bufsize = 1;

	int i,j;

	if ((buf = malloc(1)) == NULL) {
		perror("malloc");
		return NULL;
	}

	APPEND('[');
	for (i = 0; i < kvv->kv_pairs; i++) {
		char *key, *value;
		int key_len, value_len;
		key = kvv->kv[i].key;
		key_len = kvv->kv[i].key_len;
		value = kvv->kv[i].value;
		value_len = kvv->kv[i].value_len;
		if( i != 0 ) APPEND(',');
		APPEND('['); APPEND('"');
		for(j=0;j<key_len;j++) {
			switch( key[j] ) {
			case '\0': APPEND('\\'); APPEND('0');  break;
			case '\n': APPEND('\\'); APPEND('n');  break;
			case '"':  APPEND('\\'); APPEND('"');  break;
			case '\\': APPEND('\\'); APPEND('\\'); break;
			default:
				APPEND(key[j]);
				break;
			}
		}
		APPEND('"'); APPEND(','); APPEND('"');
		for(j=0;j<value_len;j++) {
			switch( value[j] ) {
			case '\0': APPEND('\\'); APPEND('0');  break;
			case '\n': APPEND('\\'); APPEND('n');  break;
			case '"':  APPEND('\\'); APPEND('"');  break;
			case '\\': APPEND('\\'); APPEND('\\'); break;
			default:
				APPEND(value[j]);
				break;
			}
		}
		APPEND('"'); APPEND(']');

	}
	APPEND(']'); APPEND('\0');
	return buf;
}

static char *expect_string( char **inptr, int *length ) {
	char *buf = NULL;
	unsigned int ptr = 0;
	unsigned int bufsize = 1;
	if( **inptr != '"' )
		return NULL;
	(*inptr)++;

	if ((buf = malloc(1)) == NULL) {
		perror("malloc");
		return NULL;
	}

	while( **inptr != '\0' && **inptr != '"' ) {
		if( **inptr == '\\' ) {
			(*inptr)++;
			if( **inptr == '\0' ) {
				my_free(buf);
				return NULL;
			}
			if( **inptr == 'n' ) {
				APPEND('\n');
			} else if( **inptr == '0' ) {
				APPEND('\0');
			} else {
				APPEND(**inptr);
			}
		} else {
			APPEND(**inptr);
		}
		(*inptr)++;
	}
	if( **inptr != '"' ) {
		my_free(buf);
		return NULL;
	}
	(*inptr)++;
	APPEND('\0');
	*length = ptr-1;
	return buf;
}

#define FAIL_EXIT do { kvvec_destroy(kvv, KVVEC_FREE_ALL); return NULL; } while(0)

struct kvvec *json_to_kvvec( char *inbuf ) {
	struct kvvec *kvv = kvvec_create(10);

	char *inptr = inbuf;

	while( *inptr == ' ' ) inptr++;

	if( *inptr != '[' ) FAIL_EXIT;
	inptr++;

	while( *inptr == '[' ) {
		inptr++;

		int key_len = 0;
		char *key = expect_string(&inptr, &key_len);
		if( !key ) FAIL_EXIT;

		if( *inptr != ',' ) FAIL_EXIT;
		inptr++;

		int value_len = 0;
		char *value = expect_string(&inptr, &value_len);
		if( !value ) FAIL_EXIT;

		kvvec_addkv_wlen(kvv, key, key_len, value, value_len);
		if( *inptr != ']' ) FAIL_EXIT;
		inptr++;

		if( *inptr == ',' ) inptr++;
	}
	if( *(inptr++) != ']' ) FAIL_EXIT;

	return kvv;
}
