/*
 * temp_objects.c
 *
 *  Created on: Mar 6, 2013
 *      Author: msikstrom
 */

#include <nagios/nagios.h> /* To make NSCORE defined, when loading objects.h */
#include <nagios/objects.h>
#include <stdlib.h>
#include <string.h>

#include <nagios/lib/kvvec.h>
#include "temp_objects.h"

/* Create a prototype for a given object */
#define OBJ_PROTOTYPE(_OBJ) \
		static struct _OBJ *temp_create_ ## _OBJ(struct kvvec *kvv);

/* Start of a kvvec -> object converter function */
#define INIT_OBJ_DEF(_OBJ) \
	static struct _OBJ *temp_create_ ## _OBJ(struct kvvec *kvv) { \
		struct _OBJ *obj = calloc(1,sizeof(struct _OBJ)); \
		int def = 0;         \
		int i;               \
		char *key;           \
		char *value;         \
		if( obj == NULL ) return NULL; \
		for (i = 0; i < kvv->kv_pairs; i++) { \
			key = kvv->kv[i].key; \
			value = kvv->kv[i].value;


/* Define a parameter in an object */
#define GET_ARG_ADD(_OBJ, _VAR, _FNC) \
		if( 0==strcmp(#_OBJ "." #_VAR, key) ) { \
			obj->_VAR = _FNC(value, obj->_VAR); \
			def = 1;                            \
		}

/* Define a parameter in an object */
#define GET_ARG(_OBJ, _VAR, _FNC) \
		if( 0==strcmp(#_OBJ "." #_VAR, key) ) { \
			obj->_VAR = _FNC(value);            \
			def = 1;                            \
		}

/* Finish a kvvec -> object converter function */
#define FINISH_OBJ_DEF \
		} \
		if( !def ) {     \
			free( obj ); \
			return NULL; \
		}                \
		return obj;      \
	}

static char *kvv_get_value(struct kvvec *kvv, char *key);

OBJ_PROTOTYPE( contact )
OBJ_PROTOTYPE( contactgroup )
OBJ_PROTOTYPE( host )
OBJ_PROTOTYPE( service )
OBJ_PROTOTYPE( hostgroup )
OBJ_PROTOTYPE( servicegroup )
OBJ_PROTOTYPE( command )

/*
 * Return a value from a kvvec with a given key
 */
static char *kvv_get_value(struct kvvec *kvv, char *key) {
	int i;
	for (i = 0; i < kvv->kv_pairs; i++) {
		if(0==strcmp(kvv->kv[i].key, key)) {
			return kvv->kv[i].value;
		}
	}
	return NULL;
}

static customvariablesmember *unpack_custom_var( char *customvars, customvariablesmember *prev ) {
	customvariablesmember *cvar;
	char *delim = strchr( customvars, '=' );
	int name_len;

	if( delim == NULL ) {
		/* Could not parse, skip */
		return prev;
	}
	name_len = delim - customvars;

	cvar = calloc(1,sizeof(customvariablesmember));
	cvar->next = prev;
	cvar->variable_name = strndup(customvars, name_len); /* Only copy name */
	if( !cvar->variable_name ) {
		free( cvar );
		/* FIXME: error handling */
		return prev;
	}
	cvar->variable_value = strdup(delim + 1);
	if( !cvar->variable_value ) {
		free( cvar->variable_name );
		free( cvar );
		/* FIXME: error handling */
		return prev;
	}
	return cvar;
}

static void free_custom_vars( customvariablesmember *cvar ) {
	customvariablesmember *tmp;
	while( cvar ) {
		tmp = cvar->next;

		free( cvar->variable_name );
		free( cvar->variable_value );
		free( cvar );

		cvar = tmp;
	}
}


INIT_OBJ_DEF(contact)
	GET_ARG( contact, name,                                 )
	GET_ARG( contact, alias,                                )
	GET_ARG( contact, email,                                )
	GET_ARG( contact, pager,                                )
//char    *address[MAX_CONTACT_ADDRESSES];
//struct commandsmember *host_notification_commands;
//struct commandsmember *service_notification_commands;
	GET_ARG( contact, host_notification_options,            atoi ) /* unsigned int */
	GET_ARG( contact, service_notification_options,         atoi ) /* unsigned int */
	GET_ARG( contact, minimum_value,                        atoi ) /* unsigned int */
	GET_ARG( contact, host_notification_period,             )
	GET_ARG( contact, service_notification_period,          )
	GET_ARG( contact, host_notifications_enabled,           atoi )
	GET_ARG( contact, service_notifications_enabled,        atoi )
	GET_ARG( contact, can_submit_commands,                  atoi )
	GET_ARG( contact, retain_status_information,            atoi )
	GET_ARG( contact, retain_nonstatus_information,         atoi )
	GET_ARG_ADD( host, custom_variables,                    unpack_custom_var )
FINISH_OBJ_DEF

INIT_OBJ_DEF(contactgroup)
	GET_ARG( contactgroup, group_name,                      )
	GET_ARG( contactgroup, alias,                           )
//struct contactsmember *members;
FINISH_OBJ_DEF

INIT_OBJ_DEF(host)
	GET_ARG( host, name,                                    )
	GET_ARG( host, display_name,                            )
	GET_ARG( host, alias,                                   )
	GET_ARG( host, address,                                 )
	GET_ARG( host, check_command,                           )
	GET_ARG( host, initial_state,                           atoi )
	GET_ARG( host, check_interval,                          atof )
	GET_ARG( host, retry_interval,                          atof )
	GET_ARG( host, max_attempts,                            atoi )
	GET_ARG( host, event_handler,                           )
//struct contactgroupsmember *contact_groups;
//struct contactsmember *contacts;
	GET_ARG( host, notification_interval,                   atof )
	GET_ARG( host, first_notification_delay,                atof )
	GET_ARG( host, notification_options,                    atoi ) /* unsigned int */
	GET_ARG( host, hourly_value,                            atoi ) /* unsigned int */
	GET_ARG( host, notification_period,                     )
	GET_ARG( host, check_period,                            )
	GET_ARG( host, flap_detection_enabled,                  atoi )
	GET_ARG( host, low_flap_threshold,                      atof )
	GET_ARG( host, high_flap_threshold,                     atof )
	GET_ARG( host, flap_detection_options,                  atoi )
	GET_ARG( host, stalking_options,                        atoi ) /* unsigned int */
	GET_ARG( host, check_freshness,                         atoi )
	GET_ARG( host, freshness_threshold,                     atoi )
	GET_ARG( host, process_performance_data,                atoi )
	GET_ARG( host, checks_enabled,                          atoi )
//const char *check_source;
	GET_ARG( host, accept_passive_checks,                   atoi )
	GET_ARG( host, event_handler_enabled,                   atoi )
	GET_ARG( host, retain_status_information,               atoi )
	GET_ARG( host, retain_nonstatus_information,            atoi )
	GET_ARG( host, obsess,                                  atoi )
	GET_ARG( host, notes,                                   )
	GET_ARG( host, notes_url,                               )
	GET_ARG( host, action_url,                              )
	GET_ARG( host, icon_image,                              )
	GET_ARG( host, icon_image_alt,                          )
	GET_ARG( host, statusmap_image,                         )
	GET_ARG( host, vrml_image,                              )
	GET_ARG( host, have_2d_coords,                          atoi )
	GET_ARG( host, x_2d,                                    atoi )
	GET_ARG( host, y_2d,                                    atoi )
	GET_ARG( host, have_3d_coords,                          atoi )
	GET_ARG( host, x_3d,                                    atof )
	GET_ARG( host, y_3d,                                    atof )
	GET_ARG( host, z_3d,                                    atof )
	GET_ARG( host, should_be_drawn,                         atoi )
	GET_ARG_ADD( host, custom_variables,                    unpack_custom_var )
	GET_ARG( host, problem_has_been_acknowledged,           atoi )
	GET_ARG( host, acknowledgement_type,                    atoi )
	GET_ARG( host, check_type,                              atoi )
	GET_ARG( host, current_state,                           atoi )
	GET_ARG( host, last_state,                              atoi )
	GET_ARG( host, last_hard_state,                         atoi )
	GET_ARG( host, plugin_output,                           )
	GET_ARG( host, long_plugin_output,                      )
	GET_ARG( host, perf_data,                               )
	GET_ARG( host, state_type,                              atoi )
	GET_ARG( host, current_attempt,                         atoi )
	GET_ARG( host, current_event_id,                        atol ) /* unsigned long */
	GET_ARG( host, last_event_id,                           atol ) /* unsigned long */
	GET_ARG( host, current_problem_id,                      atol ) /* unsigned long */
	GET_ARG( host, last_problem_id,                         atol ) /* unsigned long */
	GET_ARG( host, latency,                                 atof )
	GET_ARG( host, execution_time,                          atof )
	GET_ARG( host, is_executing,                            atoi )
	GET_ARG( host, check_options,                           atoi )
	GET_ARG( host, notifications_enabled,                   atoi )
	GET_ARG( host, last_notification,                       atol ) /* time_t */
	GET_ARG( host, next_notification,                       atol ) /* time_t */
	GET_ARG( host, next_check,                              atol ) /* time_t */
	GET_ARG( host, should_be_scheduled,                     atoi )
	GET_ARG( host, last_check,                              atol ) /* time_t */
	GET_ARG( host, last_state_change,                       atol ) /* time_t */
	GET_ARG( host, last_hard_state_change,                  atol ) /* time_t */
	GET_ARG( host, last_time_up,                            atol ) /* time_t */
	GET_ARG( host, last_time_down,                          atol ) /* time_t */
	GET_ARG( host, last_time_unreachable,                   atol ) /* time_t */
	GET_ARG( host, has_been_checked,                        atoi )
	GET_ARG( host, is_being_freshened,                      atoi )
	GET_ARG( host, notified_on,                             atoi )
	GET_ARG( host, current_notification_number,             atoi )
	GET_ARG( host, no_more_notifications,                   atoi )
	GET_ARG( host, current_notification_id,                 atol ) /* unsigned long */
	GET_ARG( host, check_flapping_recovery_notification,    atoi )
	GET_ARG( host, scheduled_downtime_depth,                atoi )
	GET_ARG( host, pending_flex_downtime,                   atoi )
//int     state_history[MAX_STATE_HISTORY_ENTRIES];
//int     state_history_index;
	GET_ARG( host, last_state_history_update,               atol ) /* time_t */
	GET_ARG( host, is_flapping,                             atoi )
	GET_ARG( host, flapping_comment_id,                     atol ) /* unsigned long */
	GET_ARG( host, percent_state_change,                    atof )
	GET_ARG( host, total_services,                          atoi )
	GET_ARG( host, total_service_check_interval,            atol ) /* unsigned long */
	GET_ARG( host, modified_attributes,                     atol ) /* unsigned long */
//struct command *event_handler_ptr;
//struct command *check_command_ptr;
//struct timeperiod *check_period_ptr;
//struct timeperiod *notification_period_ptr;
//struct objectlist *hostgroups_ptr;
//struct objectlist *exec_deps, *notify_deps;
//struct objectlist *escalation_list;*/
FINISH_OBJ_DEF

INIT_OBJ_DEF(service)
	GET_ARG( service, host_name,                            )
	GET_ARG( service, description,                          )
	GET_ARG( service, display_name,                         )
//struct servicesmember *parents;
//struct servicesmember *children;
	GET_ARG( service, check_command,                        )
	GET_ARG( service, event_handler,                        )
	GET_ARG( service, initial_state,                        atoi )
	GET_ARG( service, check_interval,                       atof )
	GET_ARG( service, retry_interval,                       atof )
	GET_ARG( service, max_attempts,                         atoi )
	GET_ARG( service, parallelize,                          atoi )
//struct contactgroupsmember *contact_groups;
//struct contactsmember *contacts;
	GET_ARG( service, notification_interval,                atof )
	GET_ARG( service, first_notification_delay,             atof )
	GET_ARG( service, notification_options,                 atoi ) /* unsigned int */
	GET_ARG( service, stalking_options,                     atoi ) /* unsigned int */
	GET_ARG( service, hourly_value,                         atoi ) /* unsigned int */
	GET_ARG( service, is_volatile,                          atoi ) /* unsigned int */
	GET_ARG( service, notification_period,                  )
	GET_ARG( service, check_period,                         )
	GET_ARG( service, flap_detection_enabled,               atoi )
	GET_ARG( service, low_flap_threshold,                   atof )
	GET_ARG( service, high_flap_threshold,                  atof )
	GET_ARG( service, flap_detection_options,               atoi ) /* unsigned int */
	GET_ARG( service, process_performance_data,             atoi )
	GET_ARG( service, check_freshness,                      atoi )
	GET_ARG( service, freshness_threshold,                  atoi )
	GET_ARG( service, accept_passive_checks,                atoi )
	GET_ARG( service, event_handler_enabled,                atoi )
	GET_ARG( service, checks_enabled,                       atoi )
	GET_ARG( service, check_source,                         )
	GET_ARG( service, retain_status_information,            atoi )
	GET_ARG( service, retain_nonstatus_information,         atoi )
	GET_ARG( service, notifications_enabled,                atoi )
	GET_ARG( service, obsess,                               atoi )
	GET_ARG( service, notes,                                )
	GET_ARG( service, notes_url,                            )
	GET_ARG( service, action_url,                           )
	GET_ARG( service, icon_image,                           )
	GET_ARG( service, icon_image_alt,                       )
	GET_ARG_ADD( service, custom_variables,                 unpack_custom_var )
//#ifdef NSCORE
	GET_ARG( service, problem_has_been_acknowledged,        atoi )
	GET_ARG( service, acknowledgement_type,                 atoi )
	GET_ARG( service, host_problem_at_last_check,           atoi )
	GET_ARG( service, check_type,                           atoi )
	GET_ARG( service, current_state,                        atoi )
	GET_ARG( service, last_state,                           atoi )
	GET_ARG( service, last_hard_state,                      atoi )
	GET_ARG( service, plugin_output,                        )
	GET_ARG( service, long_plugin_output,                   )
	GET_ARG( service, perf_data,                            )
	GET_ARG( service, state_type,                           atoi )
	GET_ARG( service, next_check,                           atol ) /* time_t */
	GET_ARG( service, should_be_scheduled,                  atoi )
	GET_ARG( service, last_check,                           atol ) /* time_t */
	GET_ARG( service, current_attempt,                      atoi )
	GET_ARG( service, current_event_id,                     atol ) /* unsigned long */
	GET_ARG( service, last_event_id,                        atol ) /* unsigned long */
	GET_ARG( service, current_problem_id,                   atol ) /* unsigned long */
	GET_ARG( service, last_problem_id,                      atol ) /* unsigned long */
	GET_ARG( service, last_notification,                    atol ) /* time_t */
	GET_ARG( service, next_notification,                    atol ) /* time_t */
	GET_ARG( service, no_more_notifications,                atoi )
	GET_ARG( service, check_flapping_recovery_notification, atoi )
	GET_ARG( service, last_state_change,                    atol ) /* time_t */
	GET_ARG( service, last_hard_state_change,               atol ) /* time_t */
	GET_ARG( service, last_time_ok,                         atol ) /* time_t */
	GET_ARG( service, last_time_warning,                    atol ) /* time_t */
	GET_ARG( service, last_time_unknown,                    atol ) /* time_t */
	GET_ARG( service, last_time_critical,                   atol ) /* time_t */
	GET_ARG( service, has_been_checked,                     atoi )
	GET_ARG( service, is_being_freshened,                   atoi )
	GET_ARG( service, notified_on,                          atoi ) /* unsigned int */
	GET_ARG( service, current_notification_number,          atoi )
	GET_ARG( service, current_notification_id,              atol ) /* unsigned long */
	GET_ARG( service, latency,                              atof )
	GET_ARG( service, execution_time,                       atof )
	GET_ARG( service, is_executing,                         atoi )
	GET_ARG( service, check_options,                        atoi )
	GET_ARG( service, scheduled_downtime_depth,             atoi )
	GET_ARG( service, pending_flex_downtime,                atoi )
//int     state_history[MAX_STATE_HISTORY_ENTRIES];    /* flap detection */
//int     state_history_index;
	GET_ARG( service, is_flapping,                          atoi )
	GET_ARG( service, flapping_comment_id,                  atol ) /* unsigned long */
	GET_ARG( service, percent_state_change,                 atof )
	GET_ARG( service, modified_attributes,                  atol )

//struct host *host_ptr;
//struct command *event_handler_ptr;
	GET_ARG( service, event_handler_args,                   )
//struct command *check_command_ptr;
	GET_ARG( service, check_command_args,                   )
//struct timeperiod *check_period_ptr;
//struct timeperiod *notification_period_ptr;
//struct objectlist *servicegroups_ptr;
//#endif
//struct objectlist *exec_deps, *notify_deps;
//struct objectlist *escalation_list;
//struct service *next;
//struct timed_event *next_check_event;
FINISH_OBJ_DEF

INIT_OBJ_DEF(hostgroup)
	GET_ARG( hostgroup, group_name,                         )
	GET_ARG( hostgroup, alias,                              )
//struct hostsmember *members;
	GET_ARG( hostgroup, notes,                              )
	GET_ARG( hostgroup, notes_url,                          )
	GET_ARG( hostgroup, action_url,                         )
FINISH_OBJ_DEF

INIT_OBJ_DEF(servicegroup)
	GET_ARG( servicegroup, group_name,                      )
	GET_ARG( servicegroup, alias,                           )
//struct servicesmember *members;
	GET_ARG( servicegroup, notes,                           )
	GET_ARG( servicegroup, notes_url,                       )
	GET_ARG( servicegroup, action_url,                      )
FINISH_OBJ_DEF

INIT_OBJ_DEF(command)
	GET_ARG( command, command_line, )
	GET_ARG( command, name,                                 )
FINISH_OBJ_DEF

struct temp_objects *build_temp_objects( struct kvvec *kvv ) {
	struct temp_objects *objs = calloc(1,sizeof(struct temp_objects));
	char *tmp;

	if( objs == NULL )
		return NULL;

	objs->ctc = temp_create_contact(kvv);
	objs->cg  = temp_create_contactgroup(kvv);
	objs->hst = temp_create_host(kvv);
	objs->svc = temp_create_service(kvv);
	objs->hg  = temp_create_hostgroup(kvv);
	objs->sg  = temp_create_servicegroup(kvv);
	objs->cmd = temp_create_command(kvv);

	/* Set up relations */
	if( objs->hst && objs->svc ) {
		objs->hst->services = calloc(1,sizeof(servicesmember));
		if( objs->hst->services ) {
			objs->hst->services->host_name = objs->svc->host_name;
			objs->hst->services->service_description = objs->svc->description;
			objs->hst->services->service_ptr = objs->svc;
		}
		objs->svc->host_ptr = objs->hst;
	}
	if( objs->hst && objs->hg ) {
		objs->hg->members = calloc(1,sizeof(hostsmember));
		if( objs->hg->members ) {
			objs->hg->members->host_name = objs->hst->name;
			objs->hg->members->host_ptr = objs->hst;
		}
		objs->hst->hostgroups_ptr = calloc(1,sizeof(objectlist));
		if( objs->hst->hostgroups_ptr ) {
			objs->hst->hostgroups_ptr->object_ptr = (void*)objs->hg;
		}
	}
	if( objs->svc && objs->sg ) {
		objs->sg->members = calloc(1,sizeof(servicesmember));
		if( objs->sg->members ) {
			objs->sg->members->host_name = objs->svc->host_name;
			objs->sg->members->service_description = objs->svc->description;
			objs->sg->members->service_ptr = objs->svc;
		}
		objs->svc->servicegroups_ptr = calloc(1,sizeof(objectlist));
		if( objs->svc->servicegroups_ptr ) {
			objs->svc->servicegroups_ptr->object_ptr = (void*)objs->sg;
		}
	}
	if( objs->ctc && objs->cg ) {
		objs->cg->members = calloc(1,sizeof(contactsmember));
		if( objs->cg->members ) {
			objs->cg->members->contact_name = objs->ctc->name;
			objs->cg->members->contact_ptr = objs->ctc;
		}
		objs->ctc->contactgroups_ptr = calloc(1,sizeof(objectlist));
		if( objs->ctc->contactgroups_ptr ) {
			objs->ctc->contactgroups_ptr->object_ptr = (void*)objs->cg;
		}
	}
	objs->runcommand = kvv_get_value(kvv,"runcmd");

	if( objs->svc ) {
		objs->timeout = service_check_timeout;
	} else {
		objs->timeout = host_check_timeout;
	}
	tmp = kvv_get_value(kvv,"timeout");
	if( tmp ) objs->timeout = atoi( tmp );

	return objs;
}
void destroy_temp_objects( struct temp_objects *objs ) {
	if( objs->hst ) {
		free_custom_vars( objs->hst->custom_variables );
		free( objs->hst->services );
		free( objs->hst->hostgroups_ptr );
	}
	if( objs->hg ) {
		free( objs->hg->members );
	}
	if( objs->svc ) {
		free_custom_vars( objs->svc->custom_variables );
		free( objs->svc->servicegroups_ptr );
	}
	if( objs->sg ) {
		free( objs->sg->members );
	}
	if( objs->ctc ) {
		free_custom_vars( objs->ctc->custom_variables );
		free( objs->ctc->contactgroups_ptr );
	}
	if( objs->cg ) {
		free( objs->cg->members );
	}
	free(objs->ctc);
	free(objs->cg);
	free(objs->hg);
	free(objs->hst);
	free(objs->sg);
	free(objs->svc);
	free(objs->cmd);
	/* Don't free runcommand, that's referenced from kvv */
	free(objs);
}
